#!/usr/bin/perl
use strict;
use warnings;
use Test::More;
use FindBin;
use lib "$FindBin::Bin/../lib";
use Config::JFDI;

my $config = Config::JFDI->new( name => 'WebApp', path => "$FindBin::Bin/.." )->get;
my $dsn = "$config->{'Model::DB'}{args}{dsn}_test";

use_ok( 'StandaloneApp2', 'loading StandaloneApp2' );
use StandaloneApp2;

my $app = StandaloneApp2->new( { dsn => $dsn } );

#Drop the table if exists:
$app->dbh->do( "drop table if exists persons" );

#Create the table if it doesn't exist
$app->dbh->do( "create table if not exists persons(
    id integer primary key autoincrement,
    first_name text,
    last_name text,
    email text
)" );

$app->dbh->do( "delete from persons" );

#Add the first record in the database
$app->add( { first_name => 'John', last_name => 'Smith', email => 'j@host.com' } );

my $lines = $app->retrieve;

is( $lines->[0]{first_name}, 'John', 'First first name' );
is( $lines->[0]{last_name}, 'Smith', 'First last name' );
is( $lines->[0]{email}, 'j@host.com', 'First email' );

#Add the second record in the database
$app->add( { first_name => 'Ion', last_name => 'Pop', email => 'ion@pop.ro' } );

$lines = $app->retrieve;

is( $lines->[0]{first_name}, 'John', 'First first name' );
is( $lines->[0]{last_name}, 'Smith', 'First last name' );
is( $lines->[0]{email}, 'j@host.com', 'First email' );

is( $lines->[1]{first_name}, 'Ion', 'Second first name' );
is( $lines->[1]{last_name}, 'Pop', 'Second last name' );
is( $lines->[1]{email}, 'ion@pop.ro', 'Second email' );

#Add the third record in the database
$app->add( { first_name => 'Ivan', last_name => 'Ivanov', email => 'i@host.ru' } );

$lines = $app->retrieve;

is( $lines->[0]{first_name}, 'John', 'First first name' );
is( $lines->[0]{last_name}, 'Smith', 'First last name' );
is( $lines->[0]{email}, 'j@host.com', 'First email' );

is( $lines->[1]{first_name}, 'Ion', 'Second first name' );
is( $lines->[1]{last_name}, 'Pop', 'Second last name' );
is( $lines->[1]{email}, 'ion@pop.ro', 'Second email' );

is( $lines->[2]{first_name}, 'Ivan', 'Third first name' );
is( $lines->[2]{last_name}, 'Ivanov', 'Third last name' );
is( $lines->[2]{email}, 'i@host.ru', 'Third email' );


#Change the fields in the second record
$app->edit( 2, { first_name => 'Radu', last_name => 'Crisan', email => 'r@eml.ro' } );

$lines = $app->retrieve;

is( $lines->[1]{first_name}, 'Radu', 'Second first name changed' );
is( $lines->[1]{last_name}, 'Crisan', 'Second last name changed' );
is( $lines->[1]{email}, 'r@eml.ro', 'Second email changed' );

#Change the fields in the first record
$app->edit( 1, { first_name => 'John', last_name => 'Wayne', email => 'w@email.com' } );

$lines = $app->retrieve;

is( $lines->[0]{first_name}, 'John', 'First first name changed' );
is( $lines->[0]{last_name}, 'Wayne', 'First last name changed' );
is( $lines->[0]{email}, 'w@email.com', 'First email changed' );


#Delete the second record
$app->delete( 2 );

$lines = $app->retrieve;

is( $lines->[0]{first_name}, 'John', 'First first name after first delete' );
is( $lines->[0]{last_name}, 'Wayne', 'First last name after first delete' );
is( $lines->[0]{email}, 'w@email.com', 'First email after first delete' );

is( $lines->[1]{first_name}, 'Ivan', 'Second first name after first delete' );
is( $lines->[1]{last_name}, 'Ivanov', 'Second last name after first delete' );
is( $lines->[1]{email}, 'i@host.ru', 'Second email after first delete' );

#Delete the first record
$app->delete( 1 );

$lines = $app->retrieve;

is( $lines->[0]{first_name}, 'Ivan', 'Single first name after second delete' );
is( $lines->[0]{last_name}, 'Ivanov', 'Single last name after second delete' );
is( $lines->[0]{email}, 'i@host.ru', 'Single email after second delete' );


#Add a new record
$app->add( { first_name => 'Giani', last_name => 'Nanini', email => 'j@eml.it' } );

$lines = $app->retrieve;

is( $lines->[0]{first_name}, 'Ivan', 'First first name after new addition' );
is( $lines->[0]{last_name}, 'Ivanov', 'First last name after new addition' );
is( $lines->[0]{email}, 'i@host.ru', 'First email after new addition' );

is( $lines->[1]{first_name}, 'Giani', 'Second first name after new addition' );
is( $lines->[1]{last_name}, 'Nanini', 'Second last name after new addition' );
is( $lines->[1]{email}, 'j@eml.it', 'Second email after new addition' );


#Retrieve just a single member
$lines = $app->retrieve( 4 );

is( $lines->[0]{first_name}, 'Giani', 'Single first name retrieved' );
is( $lines->[0]{last_name}, 'Nanini', 'Single last name retrieved' );
is( $lines->[0]{email}, 'j@eml.it', 'Single email retrieved' );


#Delete the table
$app->dbh->do( "drop table if exists persons" );

done_testing();
